/**
	$Id: InfoRequestHandler.java 6444 2014-04-09 12:21:20Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.jobagent.handlers;

import java.net.Socket;

import com.jezhumble.javasysmon.CpuTimes;
import com.jezhumble.javasysmon.JavaSysMon;
import com.jezhumble.javasysmon.MemoryStats;

import net.sf.basedb.clients.jobagent.Agent;
import net.sf.basedb.util.jobagent.JobAgentInfo;
import net.sf.basedb.util.jobagent.RequestHandler;

/**
	This is a request handler for the <code>info</code> and <code>status</code>
	commands. It creates a {@link JobAgentInfo} object and uses it's
	<code>toString()</code> method to return the answer.
 
	@author nicklas
	@version 2.0
	@base.modified $Date: 2014-04-09 14:21:20 +0200 (Wed, 09 Apr 2014) $
*/
public class InfoRequestHandler
	implements RequestHandler
{

	private static boolean useJavaSysMon = true;
	
	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(InfoRequestHandler.class);
	
	private final Agent agent;
	public InfoRequestHandler(Agent agent)
	{
		this.agent = agent;
	}
	
	/*
		From the RequestHandler interface
		-------------------------------------------
	*/
	@Override
	public String handleCmd(Socket incoming, String cmd)
	{
		String answer = null;
		if ("info".equals(cmd))
		{
			Integer cpu = null;
			Long totalMemory = null;
			Long usedMemory = null;
			if (useJavaSysMon)
			{
				try
				{
					JavaSysMon sysMon = new JavaSysMon();
					MemoryStats mem = sysMon.physical();
					totalMemory = mem.getTotalBytes();
					usedMemory = mem.getTotalBytes() - mem.getFreeBytes();
					CpuTimes t1 = sysMon.cpuTimes();
					try
					{
						Thread.sleep(1000);
					}
					catch (InterruptedException ex)
					{}
					CpuTimes t2 = sysMon.cpuTimes();
					long total = t2.getTotalMillis() - t1.getTotalMillis();
					long idle = t2.getIdleMillis() - t1.getIdleMillis();
					if (total > 0)
					{
						cpu = (int)(100*(total-idle) / total);
					}
				}
				catch (RuntimeException ex)
				{
					useJavaSysMon = false;
					log.info("Could not get cpu/memory information. JavaSysMon is now disabled.", ex);
				}
			}
			JobAgentInfo info = new JobAgentInfo(!agent.isRunning(), cpu, usedMemory, totalMemory, agent.getRunningJobs());
			answer = "OK\n"+info.toString();
		}
		else if ("status".equals(cmd))
		{
			JobAgentInfo info = new JobAgentInfo(!agent.isRunning(), null, null, null, null);
			answer = "OK\n"+info.toString();
		}
		else
		{
			answer = "FAILED Unhandled command: " + cmd;
		}
		return answer;
	}
	// -------------------------------------------

}
