/**
	$Id$

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.list;

import net.sf.basedb.core.ItemContext;
import net.sf.basedb.util.extensions.AbstractRenderer;
import net.sf.basedb.util.extensions.Extension;
import net.sf.basedb.util.formatter.Formatter;

/**
	Renderer implementation that add formatter and data loader
	implementation for a list column that can be exported.
	The renderer must be created with a reference to the {@link ItemContext}
	that is used by the table exporter. If the {@link ListColumnAction#getExportFormatter()}
	returns a non-null object, it is stored in the context under the
	key <code>export.formatter.<i>exportProperty</i></code>. The
	renderer also set the <code>export.dataloader.<i>exportProperty</i></code>
	to a {@link ListColumnDataLoader} instance which simply forwards the
	call to {@link ListColumnAction#getExportValue(net.sf.basedb.core.DbControl, Object)}.
	
	@author nicklas
	@since 3.2
*/
public class ListColumnExportRenderer<I>
	extends AbstractRenderer<ListColumnAction<I, ?>> 
{

	private final ItemContext exportContext;
	
	public ListColumnExportRenderer(ItemContext exportContext)
	{
		this.exportContext = exportContext;
	}

	/*
		From the Renderer interface
		---------------------------
	*/
	@Override
	public void render(ListColumnAction<I, ?> action, Extension<? extends ListColumnAction<I, ?>> ext) 
	{
		if (action.isExportable())
		{
			String exportProperty = action.getExportProperty();
			if (exportProperty == null) exportProperty = action.getProperty();
			Formatter<?> f = action.getExportFormatter();
			if (f != null) 
			{
				exportContext.setObject("export.formatter."+exportProperty, f);
			}
			exportContext.setObject("export.dataloader."+exportProperty, new ListColumnDataLoader<I>(action));
		}
	}
	// -----------------------------
	
}
