package net.sf.basedb.clients.web.extensions.list;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import net.sf.basedb.clients.web.formatter.FormatterFactory;
import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.snapshot.AnnotationSnapshot;
import net.sf.basedb.core.snapshot.AnnotationTypeFilter;
import net.sf.basedb.util.Enumeration;
import net.sf.basedb.util.filter.Filter;
import net.sf.basedb.util.formatter.CollectionFormatter;
import net.sf.basedb.util.listable.SourceItemTransformer;

/**
	Related item column that load annotation values from the parent or child items.
	@author nicklas
	@since 3.16, 3.17
*/
public class RelatedItemAnnotationColumn 
	extends RelatedItemColumn
{

	private final AnnotationType at;
	private final Filter<AnnotationSnapshot> atFilter;
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	RelatedItemAnnotationColumn(DbControl dc, int index, Specification spec, RelatedItemHelper helper, AnnotationType at)
	{
		super(dc, index, spec, helper);
		this.at = at;
		this.atFilter = new AnnotationTypeFilter(at);
		setTitle(spec.generateTitle(at.getName() + " [A]"));
		setTooltip(spec.generateTooltip(at.getName()));
		setValueType(at.getValueType());
		setExportFormatter(FormatterFactory.getTypeFormatter(dc.getSessionControl(), at.getValueType()));
		if (!helper.lazy) setFormatter(new CollectionFormatter(getExportFormatter()));

		if (at.isEnumeration())
		{
			Enumeration<String, String> annotationEnum = new Enumeration<String, String>();
			annotationEnum.add("", "-none-");
			List<?> values = at.getValues();
			for (Object value : values)
			{
				String encoded = HTML.encodeTags(value.toString());
				annotationEnum.add(encoded, encoded);
			}
			setEnumeration(annotationEnum);
		}
	}
	
	@Override
	public Object getValue(RelatedItemHelper helper, Annotatable item, SourceItemTransformer preTransform) 
	{
		Set<Annotatable> items = getRelatedItems(item, preTransform);
		if (items.size() == 0) return null;
		
		List<Object> values = new ArrayList<>();
		for (Annotatable p : items)
		{
			for (AnnotationSnapshot ss : helper.manager.findAnnotations(helper.dc, p, atFilter, true, false))
			{
				values.addAll(ss.getActualValues());
			}
		}
		return values.size() == 0 ? null : values;
	}

}
