/**
	$Id: SimpleProgressReporter.java 6795 2015-03-24 10:27:59Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**
 	This is a simple implementation of the {@link ProgressReporter} interface,
 	which simply remembers that last values. An application may use the {@link
 	#getMessage()} and {@link #getPercent()} method to get information about the
 	progress. This implementation also supports forwarding the progress information
 	to another progress reporter.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2015-03-24 11:27:59 +0100 (Tue, 24 Mar 2015) $
*/
public class SimpleProgressReporter
	implements ProgressReporter
{

	private final ProgressReporter progress;
	private String message;
	private int percent;
	private Throwable error;
	
	
	/**
		Create a new simple progress reporter.
		@param progress An optional progress which is also notified about
			the progress
	*/
	public SimpleProgressReporter(ProgressReporter progress)
	{
		this.progress = progress;
	}
	
	/*
		From the ProgressReporter interface
		-------------------------------------------
	*/
	@Override
	public void display(int percent, String message)
	{
		this.percent = percent;
		if (message != null) this.message = message;
		if (progress != null) progress.display(percent, message);
	}
	@Override
	public void append(String message)
	{
		this.message += message;
		if (progress != null) progress.append(message);
	}
	// -------------------------------------------

	/**
		Get the last progress message this progress reporter should display.
	*/
	public String getMessage()
	{
		return message;
	}
	/**
		Get the last percentage completed of the task.
	*/
	public int getPercent()
	{
		return percent;
	}
	
	/**
		Set an error that happened while processing some action that
		was reported via this progress reporter.
		@since 3.5
	*/
	public void setError(Throwable error)
	{
		this.percent = 100;
		this.error = error;
	}
	
	/**
		Get error that happened while processing some action that
		was reported via this progress reporter.
		@since 3.5
	*/
	public Throwable getError()
	{
		return error;
	}
}
