/*
	$Id $

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.authentication;

import java.util.HashMap;
import java.util.Map;

/**
	Class for holding extra information about a user.
	Can be used by external authentication managers (see {@link AuthenticationManager})
	to return more information about a user.
	
	@author Nicklas
	@since 3.3
*/
public class ExtraUserInformation
{
	
	private String name;
	private String organisation;
	private String address;
	private String phone;
	private String fax;
	private String email;
	private String url;
	private String description;

	private Map<String, Object> extended;
	
	/**
		Creates a new <code>ExtraUserInformation</code> object.
	*/
	public ExtraUserInformation()
	{}

	/**
		The full name of the user.
	*/
	public String getName() 
	{
		return name;
	}

	public void setName(String name) 
	{
		this.name = name;
	}

	/**
		The organisation of the user.
	*/
	public String getOrganisation() 
	{
		return organisation;
	}

	public void setOrganisation(String organisation)
	{
		this.organisation = organisation;
	}

	/**
		The postal address to the user.
	*/
	public String getAddress() 
	{
		return address;
	}

	public void setAddress(String address) 
	{
		this.address = address;
	}

	/**
		The phone number to the user.
	*/
	public String getPhone() 
	{
		return phone;
	}

	public void setPhone(String phone) 
	{
		this.phone = phone;
	}

	/**
		The fax number to the user.
	*/
	public String getFax() 
	{
		return fax;
	}

	public void setFax(String fax)
	{
		this.fax = fax;
	}

	public String getEmail() 
	{
		return email;
	}

	/**
		The email address to the user.
	*/
	public void setEmail(String email) 
	{
		this.email = email;
	}

	/**
		An URL to the user homepage (or similar).
	*/
	public String getUrl()
	{
		return url;
	}

	public void setUrl(String url) 
	{
		this.url = url;
	}

	/**
		A description of the user.
	*/
	public String getDescription() 
	{
		return description;
	}

	public void setDescription(String description)
	{
		this.description = description;
	}

	/**
		Get an extended property value.
	*/
	public Object getExtended(String name)
	{
		return extended == null ? null : extended.get(name);
	}
	
	/**
		Set the value of an extended property.
	*/
	public void setExtended(String name, Object value)
	{
		if (extended == null) extended = new HashMap<String, Object>();
		extended.put(name, value);
	}

}
