/**
	$Id: EventHandler.java 5224 2010-01-27 13:03:33Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.bfs;

/**
	Event handlers are used for reacting to various parsing events when
	parsing BFS files. Different parsers generate different types of events,
	and generally, different event handler implementations are needed
	for different BFS file types.

	@author Nicklas
	@version 2.15
	@base.modified $Date: 2010-01-27 14:03:33 +0100 (Wed, 27 Jan 2010) $
*/
public interface EventHandler
{
	
	/**
		The method is called by the parser when it has found something
		interesting in a BFS file. See the respective parser documentation
		for documentation about the events that it generates. There is
		usually some kind of information associated with the event. This
		information is sent to the event handler as an object. The parser
		should document what kind of information it sends so that event
		handler implementation can react and use the information.
		<p>
		It is recommended that event handlers ignore event types they
		don't know about.
		
		@param eventType The type of event. See the parser documentation
			for more information about the event types it generates
		@param eventData The data that is associated with the event
		@param parser The parser that is resposible for parsing the file
	*/
	public <T> void handleEvent(EventType<T> eventType, T eventData, BfsParser parser);
	
}
