/**
	$Id: ExtensionPointKey.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager;

import net.sf.basedb.util.extensions.ExtensionPoint;

/**
	Key implementation used to identify extension points.
	Since each extension point is identified by it's id,
	we use that as the key for equality.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (Mon, 20 Apr 2015) $
	@see ExtensionPoint
*/
public class ExtensionPointKey
	implements ObjectKey<ExtensionPoint<?>>
{
	
	private final String id;
	
	/**
		Create a new key for an extension point with the given id.
		@param id The id (required)
	*/
	public ExtensionPointKey(String id)
	{
		if (id == null) throw new NullPointerException("id");
		this.id = id;
	}

	/**
		Create a new key for an extension point.
		@param ep The extension point (required)
	*/
	public ExtensionPointKey(ExtensionPoint<?> ep)
	{
		this(ep.getId());
	}
	
	/*
		From the ObjectKey interface
		----------------------------
	*/
	@Override
	public String toDescription()
	{
		return "Extension point '" + id + "'";
	}
	// -----------------------------

	/*
		From the Object class
		---------------------
	*/
	@Override
	public int hashCode()
	{
		return id.hashCode();
	}

	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null || obj.getClass() != this.getClass()) return false;
		ExtensionPointKey other = (ExtensionPointKey)obj;
		return other.id.equals(this.id);
	}

	@Override
	public String toString()
	{
		return "ExtensionPointKey@" + System.identityHashCode(this) + "[" + id + "]";
	}
	// -----------------------
	
	
}
