/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.Set;

import net.sf.basedb.core.DerivedBioAssay;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	Source item transformer implementation that transform raw bioassay items 
	to derived bioassays.
	
	@author Nicklas
	@since 3.5
*/
public class RawBioAssayToDerivedBioAssayTransformer 
	extends AbstractSourceItemTransformer
{

	/**
		Create a new raw bioassay to derived bioassay transformer.
	*/
	public RawBioAssayToDerivedBioAssayTransformer() 
	{
		super(Item.RAWBIOASSAY, Item.DERIVEDBIOASSAY);
	}

	
	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		context.collect(getSourceItemType(), source);
		ItemQuery<DerivedBioAssay> query = DerivedBioAssay.getQuery();
		query.setIncludes(context.getInclude());
		query.join(Hql.innerJoin("rawBioAssays", "rba"));
		query.restrict(
			Restrictions.in(
				Hql.alias("rba"), 
				Expressions.parameter("rawBioAssays")
			)
		);
		return context.avoid(getTargetItemType(), safeIdList(context.getDbControl(), query, "rawBioAssays", source));
	}

}
