/**
	$Id: BasicItemFilter.java 4740 2009-02-05 13:39:25Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.filter;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.Item;
import net.sf.basedb.util.filter.Filter;
import net.sf.basedb.util.overview.Node;

/**
	A filter implementation that matches against the item
	that is attached to nodes. It will evaluate to true 
	for all nodes that has the same item as the reference
	item.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-02-05 14:39:25 +0100 (Thu, 05 Feb 2009) $
*/
public class BasicItemFilter
	implements Filter<Node>
{
	private final Item itemType;
	private final BasicItem item;
	
	/**
		Create a filter that finds nodes that has the given
		item attached to it.
		@param item The item to look for
	*/
	public BasicItemFilter(BasicItem item)
	{
		this.item = item;
		this.itemType = item.getType();
	}

	/*
		From the Filter interface
		-------------------------
	*/
	@Override
	public boolean evaluate(Node node)
	{
		return itemType == node.getItemType() && item.equals(node.getItem());
	}
	// -----------------------------
}
