/**
	$Id: NodeLoaderFactory.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.ItemNotFoundException;

/**
	A node loader factory is a class that know how to
	create {@link NodeLoader}:s for a given key domain.
	
	@author Nicklas
	@version 2.10
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (Fri, 21 Oct 2011) $
*/
public interface NodeLoaderFactory<L, K>
{

	/**
		Create a node loader that knows how to load nodes for items
		that are specified by the given key. The key can be almost anything.
		The interpretation of it is entirely up to the factory implementation.
		
		@param key A key that can be used to identify which items the node
			loader should load
		@return A node loader object
		@throws ItemNotFoundException If a node loader for the given key 
			can't be found. NOTE! Instead of throwing an exception it is possible
			to return a {@link NullNodeLoader} or any other "generic" implementation.
		@throws BaseException If there is any other problem creating the
			node loader
	*/
	public NodeLoader<? extends L> createNodeLoader(K key);
	
}
