/*
	$Id: TestAnnotation.java 7670 2019-03-21 12:34:37Z nicklas $
	
	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import net.sf.basedb.core.*;
import net.sf.basedb.core.Annotation.Source;
import net.sf.basedb.core.snapshot.AnnotationSnapshot;
import net.sf.basedb.core.snapshot.AnnotationTypeFilter;
import net.sf.basedb.core.snapshot.SnapshotManager;
import net.sf.basedb.util.EqualsHelper;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

public class TestAnnotation
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing annotations");
		write_header();
		
		int celsiusId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.TEMPERATURE), "C");
		int fahrenheitId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.TEMPERATURE), "F");
		int meterId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.LENGTH), "m");
		int millimeterId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.LENGTH), "mm");

		// Create annotation types and array design
		int intId = TestAnnotationType.test_create(null, Type.INT, 0, 10, null, Item.ARRAYDESIGN, 1, null, true);
		int longId = TestAnnotationType.test_create(null, Type.LONG, 0, 1056567756767l, null, Item.ARRAYDESIGN, 1, null, true);
		int floatId = TestAnnotationType.test_create("Temperature", Type.FLOAT, celsiusId, 10.1f, null, Item.ARRAYDESIGN, 1, null, true);
		int doubleId = TestAnnotationType.test_create("Size", Type.DOUBLE, meterId, 8.13454646d, null, Item.ARRAYDESIGN, 1, null, true);
		int dateId = TestAnnotationType.test_create(null, Type.DATE, 0, new Date(), null, Item.ARRAYDESIGN, 1, null, true);
		int timestampId = TestAnnotationType.test_create(null, Type.TIMESTAMP, 0, new Date(), null, Item.ARRAYDESIGN, 1, null, true);
		int textId = TestAnnotationType.test_create(null, Type.TEXT, 0, "default value", null, Item.ARRAYDESIGN, 1, null, true);
		int stringId = TestAnnotationType.test_create(null, Type.STRING, 0, "default value", null, Item.ARRAYDESIGN, 1, null, true);
		int booleanId = TestAnnotationType.test_create(null, Type.BOOLEAN, 0, true, null, Item.ARRAYDESIGN, 1, null, true);
		int enumId = TestAnnotationType.test_create(null, Type.INT, 0, 3, null, Item.ARRAYDESIGN, 1, new Object[] { 1, 2, 3, 4, 5 }, false);
		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, true);
		int arrayDesignId2 = TestArrayDesign.test_create(Platform.AFFYMETRIX, true);
		int arrayDesignId3 = TestArrayDesign.test_create(Platform.AFFYMETRIX, true);

		// Create project and a project-specific annotation
		int project1 = TestProject.test_create("Project #1", false);
		int project2 = TestProject.test_create("Project #2", false);
		int projectSpecificId = TestAnnotationType.test_create("Project data", Type.STRING, 0, null, null, Item.ARRAYDESIGN, 1, null, true);
		TestAnnotationType.test_set_project_annotations(projectSpecificId, true);
		
		// Annotate the array design and list the annotations
		int intAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, intId, 0, 10);
		int longAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, longId, 0, 1023456789012345l);
		int floatAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, floatId, celsiusId, 11.365789456857f);
		int doubleAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, doubleId, meterId, 11.365789456857d);
		int floatAnnotationId2 = test_annotatate(Item.ARRAYDESIGN, arrayDesignId2, floatId, fahrenheitId, 11.365789456857f);
		int doubleAnnotationId2 = test_annotatate(Item.ARRAYDESIGN, arrayDesignId2, doubleId, millimeterId, 11.365789456857d);
		int dateAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, dateId, 0, new Date());
		int timestampAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, timestampId, 0, new Date());
		int textAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, textId, 0, "a text annotation");
		int stringAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, stringId, 0, "a string annotation");
		int booleanAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, booleanId, 0, false);
		int enumAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, enumId, 0, 4);
		
		// Testing 'null' list and 'empty' list
		int shouldfail1 = test_annotatate(Item.ARRAYDESIGN, arrayDesignId2, stringId, 0, (Object[])null);
		int shouldfail2 = test_annotatate(Item.ARRAYDESIGN, arrayDesignId2, textId, 0, new Object[0]);
		
		test_list_annotations(Item.ARRAYDESIGN, arrayDesignId, null, 10);
		test_list_annotations(Item.ARRAYDESIGN, arrayDesignId2, Annotation.Source.PRIMARY, 2);

		// Test: inherit and clone annotations
		int rawBioAssayId = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, 0, arrayDesignId, false);
		int rawBioAssayId2 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, 0, arrayDesignId2, false);
		test_inherit_annotation(Item.RAWBIOASSAY, rawBioAssayId, intAnnotationId, false);
		int clonedId = test_inherit_annotation(Item.RAWBIOASSAY, rawBioAssayId, stringAnnotationId, true);
		test_modify_clone(clonedId, 0, "a cloned annotation");
		test_list_annotations(Item.RAWBIOASSAY, rawBioAssayId, null, 2);
		
		test_inherit_annotationset(Item.RAWBIOASSAY, rawBioAssayId, Item.ARRAYDESIGN, arrayDesignId, 8);
		test_list_annotations(Item.RAWBIOASSAY, rawBioAssayId, Annotation.Source.INHERITED, 9);
		
		// Test project specific annotations -- set a default value and a project-specific for "Project #1"
		int defaultProjectAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId2, projectSpecificId, 0, "default value");
		test_inherit_annotation(Item.RAWBIOASSAY, rawBioAssayId2, defaultProjectAnnotationId, false);
		
		TestProject.test_activate_project(project1, null);
		int project1AnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId2, projectSpecificId, 0, "project 1 value");
		test_inherit_annotation(Item.RAWBIOASSAY, rawBioAssayId2, project1AnnotationId, false);
		
		TestProject.test_noactive_project();
		
		if (defaultProjectAnnotationId == project1AnnotationId)
		{
			write("--Create project-specific annotation value FAILED (test returns the default value");
			ok = false;
		}
		
		// Check that the annotation values are as expected
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId2, projectSpecificId, "default value", "project 1 value");
		TestProject.test_activate_project(project1, null);
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId2, projectSpecificId, "project 1 value", "default value");
		TestProject.test_activate_project(project2, null);
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId2, projectSpecificId, "default value", "project 1 value");
		TestProject.test_noactive_project();
		
		// Test new API for working with project-specific values
		// Create project-specific value when no project is active
		test_annotatate_forceproject(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, project1, "project 1 value (3)");

		// Create default value when a project is active
		TestProject.test_activate_project(project1, null);
		test_annotatate_forceproject(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, 0, "default value (3)");

		// Check the values
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, "project 1 value (3)", "default value (3)");
		TestProject.test_activate_project(project2, null);
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, "default value (3)", "project 1 value (3)");
		TestProject.test_noactive_project();
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, "default value (3)", "project 1 value (3)");
		
		// Update project-specific value when no project is active
		test_annotatate_forceproject(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, project1, "project 1 value (updated)");
		// Update default value when a project is active
		TestProject.test_activate_project(project1, null);
		test_annotatate_forceproject(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, 0, "default value (updated)");
		
		// Check the values
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, "project 1 value (updated)", "default value (updated)");
		TestProject.test_activate_project(project2, null);
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, "default value (updated)", "project 1 value (updated)");
		TestProject.test_noactive_project();
		test_project_annotation(Item.ARRAYDESIGN, arrayDesignId3, projectSpecificId, "default value (updated)", "project 1 value (updated)");
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		// Remove project-specific annotation
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId2, projectSpecificId);
		TestProject.test_noactive_project();
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId2, projectSpecificId);
		
		// Remove an annotation that is inherited
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, intId);
		test_list_annotations(Item.RAWBIOASSAY, rawBioAssayId, null, 9);
		
		TestRawBioAssay.test_delete(rawBioAssayId);
		TestRawBioAssay.test_delete(rawBioAssayId2);

		// Remove the annotations
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, intId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, longId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, floatId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, doubleId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, dateId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, timestampId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, textId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, stringId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, booleanId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, enumId);
		test_list_annotations(Item.ARRAYDESIGN, arrayDesignId, null, 0);

		// Delete annotation types and array design
		TestArrayDesign.test_delete(arrayDesignId);
		TestArrayDesign.test_delete(arrayDesignId2);
		TestArrayDesign.test_delete(arrayDesignId3);
		TestAnnotationType.test_delete(projectSpecificId);
		TestAnnotationType.test_delete(intId);
		TestAnnotationType.test_delete(longId);
		TestAnnotationType.test_delete(floatId);
		TestAnnotationType.test_delete(doubleId);
		TestAnnotationType.test_delete(dateId);
		TestAnnotationType.test_delete(timestampId);
		TestAnnotationType.test_delete(textId);
		TestAnnotationType.test_delete(stringId);
		TestAnnotationType.test_delete(booleanId);
		TestAnnotationType.test_delete(enumId);
		
		TestProject.test_delete(project1);
		TestProject.test_delete(project2);

		write("++Testing annotations "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static int test_annotatate(Item itemType, int itemId, int annotationTypeId, int unitId, Object... value)
	{
		if (itemId == 0 || annotationTypeId == 0) return 0;
		int id = 0;
		boolean shouldFail = value == null || value.length == 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable annotatable = (Annotatable)itemType.getById(dc, itemId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);

			AnnotationSet as = annotatable.getAnnotationSet();
			Annotation a = as.getAnnotation(at);
			
			if (value != null)
			{
				List<Object> l = new ArrayList<Object>();
				l.addAll(Arrays.asList(value));
				Unit unit = unitId != 0 ? Unit.getById(dc, unitId) : null;
				boolean changed = a.setValuesIfDifferent(l, unit);
				if (changed && at.getProjectAnnotations() && dc.getSessionControl().getActiveProjectId() != a.getProjectId())
				{
					// We have created  a different (project-specific) annotation
					a = as.getAnnotation(at);
				}
			}
			dc.commit();
			id = a.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(a, false);
			write_item(0, a);
			if (shouldFail)
			{
				shouldFail = false;
				throw new RuntimeException("An annotation was created without values: " + value);
			}
			write("--Create annotation OK");
		}
		catch (Throwable ex)
		{
			if (shouldFail)
			{
				write("--Create annotation OK (not created: "+ex.getMessage()+")");
			}
			else
			{
				write("--Create annotation FAILED");
				ex.printStackTrace();
				ok = false;
			}
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_annotatate_forceproject(Item itemType, int itemId, int annotationTypeId, int projectId, Object... value)
	{
		if (itemId == 0 || annotationTypeId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable annotatable = (Annotatable)itemType.getById(dc, itemId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			Project project = projectId == 0 ? null : Project.getById(dc, projectId);

			AnnotationSet as = annotatable.getAnnotationSet();
			Annotation a = as.getProjectAnnotation(at, project);
			
			if (value != null)
			{
				List<Object> l = new ArrayList<Object>();
				l.addAll(Arrays.asList(value));
				boolean changed = a.setValuesIfDifferent(l, null);
			}
			dc.commit();
			id = a.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(a, false);
			write_item(0, a);
			write("--Create annotation OK");
		}
		catch (Throwable ex)
		{
			write("--Create annotation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	
	static void test_remove_annotation(Item itemType, int itemId, int annotationTypeId)
	{
		if (itemId == 0 || annotationTypeId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable annotatable = (Annotatable)itemType.getById(dc, itemId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			AnnotationSet as = annotatable.getAnnotationSet();
			as.removeAnnotation(at);
			dc.commit();
			write("--Remove annotation OK");
		}
		catch (Throwable ex)
		{
			write("--Remove annotation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_remove_all_annotations(Item itemType, int itemId)
	{
		if (itemId == 0 ) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable annotatable = (Annotatable)itemType.getById(dc, itemId);
			AnnotationSet as = annotatable.getAnnotationSet();
			for (Annotation a : as.getAnnotations(Source.PRIMARY).list(dc))
			{
				as.removeAnnotation(a.getAnnotationType());
			}
			dc.commit();
			write("--Remove all annotations OK");
		}
		catch (Throwable ex)
		{
			write("--Remove all annotations FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_annotations(Item itemType, int itemId, Annotation.Source source, int expectedResults)
	{
		DbControl dc = null;
		Annotatable at = null;
		try
		{
			dc = TestUtil.getDbControl();
			at = (Annotatable)itemType.getById(dc, itemId);
			List<Annotation> l = Collections.emptyList();
			if (at.isAnnotated())
			{
				l = at.getAnnotationSet().getAnnotations(source).list(dc);
			}
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List " + (source==null ? "ALL":source)+" annotations for " + at + " OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List " + (source==null ? "ALL":source)+" annotations for " + at + " FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_project_annotation(Item itemType, int itemId, int annotationTypeId, Object expectedValue, Object noMatchInQuery)
	{
		if (itemId == 0 || annotationTypeId == 0) return;
		DbControl dc = null;
		Annotatable item = null;
		AnnotationType at = null;
		try
		{
			dc = TestUtil.getDbControl();
			item = (Annotatable)itemType.getById(dc, itemId);
			at = AnnotationType.getById(dc, annotationTypeId);
			
			// Regular API -- check that value is the expected value
			Object value = null;
			if (item.isAnnotated())
			{
				AnnotationSet as = item.getAnnotationSet();
				if (as.hasAnnotation(at, Annotation.Source.PRIMARY))
				{
					Annotation a = as.getAnnotation(at);
					write_item(0, a);
					value = a.getValues().get(0);
				}
			}
			if (!EqualsHelper.equals(value, expectedValue))
			{
				throw new BaseException("Expected '" + expectedValue + "', got '" + value + "'");
			}
			
			// Query API -- check that the expected item is returned by the query
			ItemQuery<?> query = itemType.getQuery();
			query.restrict(new AnnotationSimpleRestriction(null, at, Operator.EQ, expectedValue, new AnnotationRestriction.Options()));
			List<?> result = new ArrayList<Object>(query.list(dc));
			if (result.size() != 1)
			{
				throw new BaseException("Query returned: " + result);
			}
			item = (Annotatable)result.get(0);
			if (item.getId() != itemId)
			{
				throw new BaseException("Query returned: "+ result);
			}
			
			// Query API -- check that the no items are returned by the query
			if (noMatchInQuery != null)
			{
				query = itemType.getQuery();
				query.restrict(new AnnotationSimpleRestriction(null, at, Operator.EQ, noMatchInQuery, new AnnotationRestriction.Options()));
				result = new ArrayList<Object>(query.list(dc));
				if (result.size() != 0)
				{
					throw new BaseException("Query returned: " + result);
				}
			}
			
			// Snapshot manager -- check that value is the expected value
			value = null;
			SnapshotManager manager = new SnapshotManager();
			List<AnnotationSnapshot> snapshots = manager.findAnnotations(dc, item, new AnnotationTypeFilter(at), true, false);
			if (snapshots.size() != 1)
			{
				throw new BaseException("SnapshotManager returned " + snapshots.size() + " annotations");
			}
			AnnotationSnapshot snapshot = snapshots.get(0);
			value = snapshot.getActualValues().get(0);
			if (!EqualsHelper.equals(value, expectedValue))
			{
				throw new BaseException("Expected '" + expectedValue + "', got '" + value + "'");
			}
			
			write("--Project annotation for " + at + " OK ("+expectedValue+")");
		}
		catch (Throwable ex)
		{
			write("--Project annotation for " + at + " FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tAnnotation type\tAnnotation set\tSource\tProject\tParent\tValues\tUnit\tRaw values");
			write("-- \t-- \t---------------\t--------------\t------\t-------\t------\t------\t----\t----------");
		}
	}
	static void write_item(int i, Annotation a)
		throws BaseException
	{
		if (TestUtil.getSilent()) return;
		
		List<?> values = null;
		if (a.getSource() == Annotation.Source.PRIMARY)
		{
			values = a.getValues(null);
		}
		else
		{
			values = a.getInheritedFrom().getValues(null);
		}
		write(i+":\t"+a.getId()+"\t"+a.getAnnotationType()+"\t"+a.getAnnotationSet()+"\t"+
				a.getSource()+"\t"+a.getProjectId()+"\t"+a.getInheritedFrom()+"\t" +
				values + "\t" + a.getUnit() + "\t" + a.getValues());
	}
	
	static void write_item(int i, AnnotationSet as)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+as.getId()+"\t"+as.getItem());
	}

	static void write(String message)
	{
		System.out.println(message);
	}
	
	static int test_inherit_annotation(Item itemType, int itemId, int annotationId, boolean clone)
	{
		if (itemId == 0 || annotationId == 0) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable annotatable = (Annotatable)itemType.getById(dc, itemId);
			Annotation a = Annotation.getById(dc, annotationId);
			AnnotationSet as = annotatable.getAnnotationSet();
			Annotation inherited = as.inheritAnnotation(a, clone);
			dc.commit();
			id = inherited.getId();
			dc = TestUtil.getDbControl();
			inherited = Annotation.getById(dc, id);
			write_item(0, inherited);
			write("--" + (clone ? "Clone" : "Inherit")+ " annotation OK");
		}
		catch (Throwable ex)
		{
			write("--" + (clone ? "Clone" : "Inherit")+ " annotation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_inherit_annotationset(Item toItemType, int toItemId, Item fromItemType, int fromItemId, int expectAdded)
	{
		if (toItemId == 0 || fromItemId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable to = (Annotatable)toItemType.getById(dc, toItemId);
			Annotatable from = (Annotatable)fromItemType.getById(dc, fromItemId);
			AnnotationSet as = to.getAnnotationSet();
			AnnotationSet inherited = from.getAnnotationSet();
			int added = as.inheritAll(inherited, false);
			if (added != expectAdded)
			{
				throw new BaseException("Expected "+expectAdded+" results, not "+added);
			}
			dc.commit();
			write("--Inherit annotation set OK (" + added + " added)");
		}
		catch (Throwable ex)
		{
			write("--Inherit annotation set FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_modify_clone(int clonedId, int unitId, Object... value)
	{
		if (clonedId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotation a = Annotation.getById(dc, clonedId);
			
			List<Object> l = new ArrayList<Object>();
			l.addAll(Arrays.asList(value));
			Unit unit = unitId != 0 ? Unit.getById(dc, unitId) : null;
			a.setValuesIfDifferent(l, unit);
			dc.commit();
			dc = TestUtil.getDbControl();
			a = Annotation.getById(dc, clonedId);
			write_item(0, a);
			write("--Modify cloned annotation OK");
		}
		catch (Throwable ex)
		{
			write("--Modify cloned annotation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}
