/*
	$Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DiskUsage;
import net.sf.basedb.core.User;
import net.sf.basedb.core.ItemResultList;

public class TestDiskUsage
{

	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing disk usage");
		write_header();

		// Standard tests: only list is meaningful
		int file_id = TestFile.test_create("test.upload.txt", false, false);
		test_list();
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestFile.test_delete(file_id);

		// Standard test: Delete
		write("++Testing disk usage " + (ok ? "OK" : "Failed") + "\n");
		return ok;
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User user = User.getById(dc, TestUtil.getSessionControl().getLoggedInUserId());
			ItemResultList<DiskUsage> l = DiskUsage.getQuery(user, null).list(dc);
			for (int i = 0; i < l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List disk usage OK (" + l.size() + ")");
		}
		catch (Throwable ex)
		{
			write("--List disk usage FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null)
				dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tItem\tQuotaType\tLocation\tBytes\tUser");
			write("-- \t-- \t----\t---------\t--------\t-----\t----");
		}
	}

	static void write_item(int i, DiskUsage du)
		throws BaseException
	{
		if (!TestUtil.getSilent())
			write(i + ":\t" + du.getId() + "\t" + du.getItem() + "\t" + du.getQuotaType() + "\t" + du.getLocation() + "\t" + du.getBytes() + "\t" + du.getUser());
	}

	static void write(String message)
	{
		System.out.println(message);
	}
}
