import net.sf.basedb.util.encode.EncoderDecoder;
import net.sf.basedb.util.encode.TabCrLfEncoderDecoder;
import net.sf.basedb.util.encode.ToSpaceEncoderDecoder;

/*
	$Id $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

public class TestEncoderDecoder
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing string encode/decode");
		EncoderDecoder tabCrLf = new TabCrLfEncoderDecoder(false);
		test_code(tabCrLf, "nothingtocode", "nothingtocode");
		test_code(tabCrLf, "a\tb\nc\rd\\e", "a\\tb\\nc\\rd\\\\e");
		
		EncoderDecoder space = new ToSpaceEncoderDecoder();
		test_code(space, "nothingtocode", "nothingtocode");
		test_code(space, "a\tb\nc\rd\\e", "a b c d\\e");

		write("++Testing string encode/decode "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_code(EncoderDecoder coder, String original, String encoded)
	{
		try
		{
			String tmp = coder.encode(original);
			if (!tmp.equals(encoded)) 
			{
				throw new Exception("Encoding of '" + original + "' failed; got '" + 
					tmp + "'; expected '" + encoded + "'");
			}
			if (coder.isSymmetrical())
			{
				tmp = coder.decode(encoded);
				if (!tmp.equals(original))
				{
					throw new Exception("Decoding of '" + encoded + "' failed; got '" + 
						tmp + "'; expected '" + original + "'");
				}
			}
			write("--Encode/decode: "+original + "<-->" + encoded + " OK");
		}
		catch (Throwable ex)
		{
			ex.printStackTrace();
			ok = false;
		}
	}
	


	static void write(String message)
	{
		System.out.println(message);
	}
	
}

