/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Platforms = function()
{
	var platforms = {};
	
	/**
		Initialize the page.
	*/
	platforms.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			Buttons.addClickHandler('btnSave', platforms.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Tab validation
			TabControl.addTabValidator('settings.info', platforms.validatePlatform);
			
			// File-only
			if (Doc.element('fileOnlyNo'))
			{
				Events.addEventHandler('fileOnlyNo', 'click', platforms.fileOnlyOnClick);
				Events.addEventHandler('fileOnlyYes', 'click', platforms.fileOnlyOnClick);
				platforms.fileOnlyOnClick();
				
				Events.addEventHandler('channels', 'keypress', Events.integerOnly);
			}
			
			// Data file types
			Events.addEventHandler('fileTypes', 'change', platforms.fileTypesOnChange);
			Buttons.addClickHandler('btnAddFileTypes', platforms.addFileTypesOnClick);
			Events.addEventHandler('btnAddFileTypes', 'base-selected', platforms.fileTypeSelected);
			Buttons.addClickHandler('required', platforms.requiredOnClick);
			Buttons.addClickHandler('multiple', platforms.requiredOnClick);
			
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'PLATFORM', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);	
			Buttons.addClickHandler('btnNewVariant', platforms.newVariant, attributes);

			TabControl.addTabActivateListener('main.variants', platforms.viewVariants);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'PLATFORM'};
			var tableAttributes = {'table-id': 'platforms'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	// Add event handlers to the 'new variant' icons
	platforms.initElements = function(element, autoInit)
	{
		if (autoInit == 'new-variant')
		{
			Buttons.addClickHandler(element, platforms.newVariant);
		}
	}
	
	/*
		Create a new variant of the current platform
	*/
	platforms.newVariant = function(event)
	{
		var platformId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('PLATFORMVARIANT', '&platform_id='+platformId);
	}
	
	platforms.viewVariants = function()
	{
		var platformId = Data.get('page-data', 'item-id');
		var url = 'variants/index.jsp?ID='+App.getSessionId();
		url += '&cmd=List&platform_id='+platformId;
		location.replace(url);
	}

	platforms.validatePlatform = function()
	{
		var frm = document.forms['platform'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (frm.externalId)
		{
			var fileOnly = Forms.getCheckedRadio(frm.fileOnly).value == '1';
			if (Strings.trim(frm.externalId.value) == '')
			{
				Forms.showNotification(frm.externalId, 'You must enter an external ID');
				return false;
			}
			else if (fileOnly)
			{
				if (!Numbers.isInteger(frm.channels.value))
				{
					Forms.showNotification(frm.channels, 'You must enter the number of channels');
					return false;
				}
				if (parseInt(frm.channels.value) <= 0)
				{
					Forms.showNotification(frm.channels, 'The number of channels must be &gt;0');
					return false;
				}
			}
		}
		return true;
	}

	platforms.save = function()
	{
		var frm = document.forms['platform'];
		if (TabControl.validateActiveTab('settings'))
		{
			Link.exportActions('fileTypes');
			frm.submit();
		}

	}
	
	platforms.fileOnlyOnClick = function()
	{
		var frm = document.forms['platform'];
		var fileOnly = Forms.getCheckedRadio(frm.fileOnly).value == '1';
		frm.rawdatatype.disabled = fileOnly;
		frm.channels.disabled = !fileOnly;
		Doc.addOrRemoveClass(frm.rawdatatype, 'required', !frm.rawdatatype.disabled);
		Doc.addOrRemoveClass(frm.channels, 'required', !frm.channels.disabled);
	}
	
	platforms.fileTypesOnChange = function()
	{
		var frm = document.forms['platform'];
		var item = frm.fileTypes[frm.fileTypes.selectedIndex].item;
		if (item && item.id)
		{
			frm.required.checked = (item.value & 1) > 0;
			frm.multiple.checked = (item.value & 2) > 0;
		}
		else
		{
			frm.required.checked = false;
			frm.multiple.checked = false;
		}
	}

	
	/**
		Open a popup dialog for selecting related file types.
	*/
	platforms.addFileTypesOnClick = function(event)
	{
		var frm = document.forms['platform'];
		var currentFileTypes = Link.getIdsInList(frm.fileTypes, 'DATAFILETYPE');
		var url = "&exclude="+currentFileTypes.join(',');
		Dialogs.selectItem('DATAFILETYPE', event.currentTarget.id, 1, url);
	}

	platforms.fileTypeSelected = function(event)
	{
		var frm = document.forms['platform'];
		var required = frm.required.checked;
		var multiple = frm.multiple.checked;
		
		var detail = event.detail;
		var newValue = 0;
		if (required) newValue += 1;
		if (multiple) newValue += 2;
		
		detail.name += required ? ' [×]' : ' [-]';
		detail.value = newValue;
		
		Link.addItem('fileTypes', 'DATAFILETYPE', event.detail);
	}

	platforms.requiredOnClick = function()
	{
		var frm = document.forms['platform'];
		var required = frm.required.checked;
		var multiple = frm.multiple.checked;
		var newValue = 0;
		if (required) newValue += 1;
		if (multiple) newValue += 2;
		for (var i = 0; i < frm.fileTypes.length; i++)	// >
		{
			var option = frm.fileTypes[i];
			if (option.selected && option.item.id)
			{
				option.item.value = newValue;
				var text = option.text.replace(/\[.*\]/, '['+(required ? '×' : '-') +']');
				option.text = text; 
			}
		}
	}

	
	return platforms;
}();

Doc.addElementInitializer(Platforms.initElements);
Doc.onLoad(Platforms.initPage);
