/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Roles = function()
{
	var roles = {};
	
	/**
		Initialize the page.
	*/
	roles.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', roles.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Permission list
			Events.addEventHandler('items', 'change', roles.itemOnChange)
			// Permission checkboxes
			Events.addEventHandler('deny', 'click', roles.permissionsOnClick);
			Events.addEventHandler('create', 'click', roles.permissionsOnClick);
			Events.addEventHandler('read', 'click', roles.permissionsOnClick);
			Events.addEventHandler('use', 'click', roles.permissionsOnClick);
			Events.addEventHandler('restricted_write', 'click', roles.permissionsOnClick);
			Events.addEventHandler('write', 'click', roles.permissionsOnClick);
			Events.addEventHandler('delete', 'click', roles.permissionsOnClick);
			Events.addEventHandler('set_owner', 'click', roles.permissionsOnClick);
			Events.addEventHandler('set_permission', 'click', roles.permissionsOnClick);

			// Tab validation
			TabControl.addTabValidator('settings.info', roles.validateRole);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'ROLE', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'ROLE'};
			var tableAttributes = {'table-id': 'roles'};
			
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}

	roles.validateRole = function()
	{
		var frm = document.forms['role'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}

	roles.save = function()
	{
		var frm = document.forms['role'];
		if (TabControl.validateActiveTab('settings'))
		{
			Link.exportActions('members');
			var systemPermission = 0;
			if (frm.share_to_everyone.checked) systemPermission += parseInt(frm.share_to_everyone.value, 10);
			if (frm.act_as_another_user.checked) systemPermission += parseInt(frm.act_as_another_user.value, 10);
			if (frm.select_jobagent.checked) systemPermission += parseInt(frm.select_jobagent.value, 10);
			frm['SYSTEM'].value = systemPermission;
			frm.submit();
		}
	}

	roles.itemOnChange = function(event)
	{
		var itemType = event.target.value;
		var frm = document.forms['role'];
		if (itemType)
		{
			var permissionCode = frm[itemType].value;
			var definedPermissions = parseInt(frm[itemType+'_defined'].value);
			frm['deny'].checked = (permissionCode & frm['deny'].value);
			frm['create'].checked = (permissionCode & frm['create'].value);
			frm['create'].disabled = !(definedPermissions & frm['create'].value);
			frm['read'].checked = (permissionCode & frm['read'].value);
			frm['read'].disabled = !(definedPermissions & frm['read'].value);
			frm['use'].checked = (permissionCode & frm['use'].value);
			frm['use'].disabled = !(definedPermissions & frm['use'].value);
			frm['restricted_write'].disabled = !(definedPermissions & frm['restricted_write'].value);
			frm['restricted_write'].checked = (permissionCode & frm['restricted_write'].value);
			frm['write'].checked = (permissionCode & frm['write'].value);
			frm['write'].disabled = !(definedPermissions & frm['write'].value);
			frm['delete'].checked = (permissionCode & frm['delete'].value);
			frm['delete'].disabled = !(definedPermissions & frm['delete'].value);
			frm['set_owner'].checked = (permissionCode & frm['set_owner'].value);
			frm['set_owner'].disabled = !(definedPermissions & frm['set_owner'].value);
			frm['set_permission'].checked = (permissionCode & frm['set_permission'].value);
			frm['set_permission'].disabled = !(definedPermissions & frm['set_permission'].value);
		}
	}
	
	roles.permissionsOnClick = function(event)
	{
		var frm = document.forms['role'];
		roles.correctPermissions(event.target.checked, event.target.name == 'deny');
		var permissionCode = roles.getPermissionCode();

		for (var i = 0; i < frm.items.length; i++)
		{
			var item = frm.items[i];
			var itemType = item.value;
			if (item.selected && itemType)
			{
				frm[itemType].value = permissionCode;
				var definedPermissions = parseInt(frm[itemType+'_defined'].value);
				var status = '';
				if (frm['deny'].checked)
				{
					status = "DENIED";
				}
				else
				{
					if (definedPermissions & frm['create'].value) status += frm['create'].checked ? "C" : "-";
					if (definedPermissions & frm['read'].value) status += frm['read'].checked ? "R" : "-";
					if (definedPermissions & frm['use'].value) status += frm['use'].checked ? "U" : "-";
					if (definedPermissions & frm['restricted_write'].value) status += frm['restricted_write'].checked ? "A" : "-";
					if (definedPermissions & frm['write'].value) status += frm['write'].checked ? "W" : "-";
					if (definedPermissions & frm['delete'].value) status += frm['delete'].checked ? "D" : "-";
					if (definedPermissions & frm['set_owner'].value) status += frm['set_owner'].checked ? "O" : "-";
					if (definedPermissions & frm['set_permission'].value) status += frm['set_permission'].checked ? "P" : "-";
				}
				var text = item.text.replace(/\[.*\]/, '['+status+']');
				item.text = text; 
			}
		}
	}
	
	roles.correctPermissions = function(checked, forceDeny)
	{
		var frm = document.forms['role'];
		if (checked)
		{
			frm['deny'].checked = frm['deny'].checked && forceDeny;
			frm['set_permission'].checked = frm['set_permission'].checked && !frm['deny'].checked;
			frm['set_owner'].checked = frm['set_owner'].checked && !frm['deny'].checked;
			frm['delete'].checked = frm['delete'].checked && !frm['deny'].checked;
			frm['write'].checked = (frm['write'].checked || frm['delete'].checked || frm['set_owner'].checked || frm['set_permission'].checked) && !frm['deny'].checked;
			frm['restricted_write'].checked = (frm['restricted_write'].checked || frm['write'].checked) && !frm['deny'].checked;
			frm['use'].checked = (frm['use'].checked || frm['write'].checked) && !frm['deny'].checked;
			frm['read'].checked = (frm['read'].checked || frm['use'].checked || frm['restricted_write'].checked) && !frm['deny'].checked;
			frm['create'].checked = frm['create'].checked && !frm['deny'].checked;
		}
		else
		{
			frm['deny'].checked = frm['deny'].checked && !frm['create'].checked && !frm['read'].checked && !frm['use'].checked && !frm['restricted_write'].checked && !frm['write'].checked && !frm['delete'].checked && !frm['set_owner'].checked && !frm['set_permission'].checked;
			frm['create'].checked = frm['create'].checked && !frm['deny'].checked;
			frm['read'].checked = frm['read'].checked && !frm['deny'].checked;
			frm['use'].checked = (frm['use'].checked && frm['read'].checked) && !frm['deny'].checked;
			frm['restricted_write'].checked = (frm['restricted_write'].checked && frm['read'].checked) && !frm['deny'].checked;
			frm['write'].checked = (frm['write'].checked && frm['use'].checked) && frm['restricted_write'].checked && !frm['deny'].checked;
			frm['delete'].checked = (frm['delete'].checked && frm['write'].checked) && !frm['deny'].checked;
			frm['set_owner'].checked = (frm['set_owner'].checked && frm['write'].checked) && !frm['deny'].checked;
			frm['set_permission'].checked = (frm['set_permission'].checked && frm['write'].checked) && !frm['deny'].checked;
		}
	}

	roles.getPermissionCode = function()
	{
		var permissionCode = 0;
		var frm = document.forms['role'];
		permissionCode += frm['deny'].checked ? parseInt(frm['deny'].value) : 0;
		permissionCode += frm['create'].checked ? parseInt(frm['create'].value) : 0;
		permissionCode += frm['read'].checked ? parseInt(frm['read'].value) : 0;
		permissionCode += frm['use'].checked ? parseInt(frm['use'].value) : 0;
		permissionCode += frm['restricted_write'].checked ? parseInt(frm['restricted_write'].value) : 0;
		permissionCode += frm['write'].checked ? parseInt(frm['write'].value) : 0;
		permissionCode += frm['delete'].checked ? parseInt(frm['delete'].value) : 0;
		permissionCode += frm['set_owner'].checked ? parseInt(frm['set_owner'].value) : 0;
		permissionCode += frm['set_permission'].checked ? parseInt(frm['set_permission'].value) : 0;
		return permissionCode;
	}

	
	return roles;
}();

Doc.onLoad(Roles.initPage);
