/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Directories = function()
{
	var directories = {};
	
	/**
		Initialize the page.
	*/
	directories.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', directories.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Tab validation
			TabControl.addTabValidator('settings.info', directories.validateDirectory);
		}
		else if (pageId == 'list-page')
		{
			IconStore.addDefaultIcons();
			IconStore.addIconSet('Search', 'search.png', 'search-selected.png');
			IconStore.addIconSet('FolderCompressed', 'folder-compressed.png', 'folder-compressed-selected.png');
			IconStore.addIconSet('FolderDeleted', 'folder-deleted.png');

			Events.addEventHandler('joust', 'joust-select', directories.onSelect);
			Events.addEventHandler('joust', 'joust-lazy-init', directories.lazyInit);
			
			Joust2.draw('joust');
			
			Buttons.addClickHandler('btnSearch', directories.searchOnClick);
			Buttons.addClickHandler('btnRefresh', directories.refresh);
			Buttons.addClickHandler('btnNewDirectory', directories.newDirectory);
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', directories.returnSelected);
			
			Joust2.select(Data.get('page-data', 'initial-directory'), !Data.int('page-data', 'sync-with-directory'));
			
			directories.initDragDrop(document);
		}
	}


	var highlightedSubdir = null;
	var highlightedElement = null;
	var hideTooltipTimer = null;
	directories.dragFiles = function(event)
	{
		if (event.dataTransfer.types.indexOf('Files') == -1) return; // There are no files so we don't care

		// Stop all default things from happening
		event.preventDefault();
		event.stopPropagation();
		
		// Find out which directory we are uploading to
		var subdir = event.currentTarget.item.path;
		if (!subdir)
		{
			event.dataTransfer.dropEffect = "none"; // No drop here!
			return;
		}
		var allowUpload = event.currentTarget.item.allowUpload;
		event.dataTransfer.dropEffect = allowUpload ? "copy" : "none";

		// Highlight the current directory if it is a sub-directory
		if (subdir != highlightedSubdir)
		{
			if (highlightedElement) 
			{
				Doc.removeClass(highlightedElement, 'drop-upload');
				Doc.removeClass(highlightedElement, 'bg-filled-50');
			}
			if (subdir)
			{
				highlightedElement = event.currentTarget;
				highlightedSubdir = subdir;
				Doc.addClass(highlightedElement, 'drop-upload');
				Doc.addClass(highlightedElement, 'bg-filled-50');
			}
			else
			{
				highlightedElement = null;
				highlightedSubdir = null;
			}
		}
		
		// Display tooltip
		var tt = Doc.element('uploadTooltip');
		if (allowUpload)
		{
			Doc.removeClass(tt, 'error');
			tt.innerHTML = 'Upload to '+subdir.replace(/\//g, '/<wbr>');
		}
		else
		{
			Doc.addClass(tt, 'error');
			tt.innerHTML = 'Not allowed to upload to '+subdir.replace(/\//g, '/<wbr>');
		}
		tt.style.left = Math.max(0, Math.min(event.clientX-tt.clientWidth/2, window.innerWidth-tt.offsetWidth))+'px';
		tt.style.top = event.clientY < window.innerHeight-30-tt.offsetHeight ? (event.clientY+30)+'px' : (event.clientY-100-tt.offsetHeight)+'px';
		Doc.show(tt);
		
		// Set a timer for hiding the tooltip since we never get any event when dragging ends
		if (hideTooltipTimer) clearTimeout(hideTooltipTimer);
		hideTooltipTimer = setTimeout(directories.hideFileUploadTooltip, 500);
	}
	
	directories.hideFileUploadTooltip = function()
	{
		if (highlightedElement) 
		{
			Doc.removeClass(highlightedElement, 'drop-upload');
			Doc.removeClass(highlightedElement, 'bg-filled-50');
			highlightedElement = null;
			highlightedSubdir = null;
		}
		Doc.hide('uploadTooltip');
	}

	var filesToUpload;
	directories.dropFiles = function(event)
	{
		if (event.dataTransfer.types.indexOf('Files') == -1) return; // There are no files so we don't care
		
		// Stop all default things from happening
		event.preventDefault();
		event.stopPropagation();

		var items = event.dataTransfer.items;
		// Check if the dropped items are files or directories
		filesToUpload = [];
		for (var i = 0; i < items.length; i++)
		{
			var fe = items[i].webkitGetAsEntry();
			if (fe.isFile || fe.isDirectory)
			{
				filesToUpload[filesToUpload.length] = fe;
			}
		}
		if (filesToUpload.length == 0) return; // Only directories

		// Open upload dialog
		var url = '../upload/drag_drop_upload.jsp?ID='+App.getSessionId();
		url += '&toDir='+event.currentTarget.id;
		url += '&numFiles='+filesToUpload.length;
		Dialogs.openPopup(url, 'DragDropUpload', 700, 450);
	}
	
	directories.getFilesToUpload = function()
	{
		return filesToUpload;
	}
	
	directories.validateDirectory = function()
	{
		var frm = document.forms['directory'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}
	
	// Submit the form
	directories.save = function()
	{
		var frm = document.forms['directory'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}
	
	/**
		Handles click event in the directory tree.
	*/
	directories.onSelect = function(event)
	{
		var menuElement = event.target;
		var menuItem = menuElement.item;

		var mode = Data.get('page-data', 'mode');
		if (mode == 'selectonedirectory')
		{
			var frm = document.forms['directory'];
			frm.directory_id.value = menuItem.id;
			frm.path.value = menuItem.path;
			//lastDirectoryId = directoryId;
		}
		else
		{
			if (menuElement.id == 'search')
			{
				var url = '../files/index.jsp?ID='+App.getSessionId();
				url += '&cmd=Search&subcontext=search';
				url += '&mode='+Data.get('page-data', 'mode');
				parent.frames['files'].location.href = url;
			}
			else
			{
				var url = '../files/index.jsp?ID='+App.getSessionId();
				url += '&cmd=List&directory_id='+menuElement.id;
				url += '&mode='+Data.get('page-data', 'mode');
				parent.frames['files'].location.href = url;
			}
		}
	}
	
	
	/**
		Lazy loading of subdirectories
	*/
	directories.lazyInit = function(event)
	{
		var menuElement = event.target;
		Doc.addClass(menuElement, 'working');
		try
		{
			var request = Ajax.getXmlHttpRequest();
			var url = 'ajax.jsp?ID='+App.getSessionId();
			url += '&cmd=GetSubdirectories&item_id=' + menuElement.id;
			request.open("GET", url, false);
			request.send(null);
			
			var response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				Forms.showNotification(menuElement.id+'-outline', response.message);
			}
			
			var childElements = Joust2.addChildren(menuElement, response.subdirs || []);
			Joust2.toggle(menuElement);
			directories.initDragDrop(childElements);
		}
		finally
		{
			Doc.removeClass(menuElement, 'working');
		}
	}
	
	// Add drag-drop event handlers to all joust items for file upload
	directories.initDragDrop = function(element)
	{
		var dirs = Doc.element(element).getElementsByClassName('joustitem');
		for (var i = 0; i < dirs.length; i++)
		{
			Events.addEventHandler(dirs[i], 'dragover', directories.dragFiles);
			Events.addEventHandler(dirs[i], 'drop', directories.dropFiles);
		}
	}

	
	directories.searchOnClick = function()
	{
		Doc.show('search');
		Joust2.select('search');
	}

	
	directories.updateSearchResults = function(files, directories)
	{
		var searchResults = Doc.element('search-text');
		if (searchResults)
		{
			searchResults.innerHTML = 'Search results (' + (files + directories) + ')';
		}

	}
	directories.refresh = function()
	{
		location.reload();
	}

	directories.synchronizeTo = function(directoryId, parentId)
	{
		var menuElement = Doc.element(directoryId);
		if (!menuElement && parentId)
		{
			var parentElement = Doc.element(parentId);
			if (parentElement && parentElement.item.isLazy)
			{
				Joust2.toggle(parentElement);
				menuElement = Doc.element(directoryId);
			}
		}
		
		if (!menuElement)
		{
			var url = '../directories/index.jsp?ID='+App.getSessionId();
			url += '&cmd=ViewItem&item_id='+directoryId;
			url += '&mode='+Data.get('page-data', 'mode');
			parent.parent.location.href = url;
			
		}
		else
		{
			Joust2.select(menuElement);
		}
	}

	directories.newDirectory = function()
	{
		var frm = document.forms['directory'];
		Items.newItem('DIRECTORY', '&parent_id='+frm.directory_id.value);
	}

	
	directories.returnSelected = function()
	{
		var frm = document.forms['directory'];
		var callback = frm.callback.value;
		var notifyTarget = window.opener.document.getElementById(callback);
		var callbackMethod = window.opener[callback];

		if (notifyTarget)
		{
			// Send event to the target in the opener window
			Events.sendCustomEvent(notifyTarget, 'base-selected', {'id': frm.directory_id.value, 'name': frm.path.value, 'itemType': 'DIRECTORY'});
		}
		else if (callbackMethod)
		{
			// Call the callback method in the opener window
			callbackMethod.call(null, frm.directory_id.value, frm.path.value);
		}
		App.closeWindow();
	}

	
	return directories;
}();

Doc.onLoad(Directories.initPage);

// For compatibility with the drag-and-drop file upload
var FileManager = function()
{
	var fileManager = {};
	
	fileManager.getFilesToUpload = function()
	{
		return Directories.getFilesToUpload();
	}
	
	return fileManager;
}();
