/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Upload = function()
{
	var upload = {};
	
	var uploadRequest;
	
	/**
		Initialize the page.
	*/
	upload.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'select-file')
		{
			// Zip formats
			upload.initZipFormats();
			Events.addEventHandler('zipUnpack', 'click', upload.unpackZipOnClick);
			
			// Upload+Close buttons
			Buttons.addClickHandler('btnUpload', upload.doUpload);
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnAbort', upload.abortUpload);

			// File
			Events.addEventHandler('the_file', 'change', upload.fileOnChange);
			
			// Error handling
			Buttons.addClickHandler('failClose', App.closeWindow);
		}
	}

	upload.validateFile = function()
	{
		var frm = document.forms['file'];
		if (Strings.trim(frm.the_file.value) == '')
		{
			Forms.showNotification(frm.the_file, 'You must select a file to upload');
			return false;
		}
		return true;
	}
	
	// Submit the form
	upload.doUpload = function()
	{
		var frm = document.forms['file'];
		if (upload.validateFile())
		{
			var upl = new FormData();
			
			// Copy form values so they are sent BEFORE the file
			upl.set('check_md5', frm.check_md5 && frm.check_md5.checked ? "1" : "0");
			upl.set('write_protected', frm.write_protected.checked ? "1" : "0");
			upl.set('replace_existing', frm.replace_existing && frm.replace_existing.checked ? "1" : "0");
			upl.set('compressed', frm.compressed && frm.compressed.value);
			if (frm.zip_format)
			{
				upl.set('zip_format', frm.zip_format.value);
				upl.set('zip_unpack', frm.zip_unpack.checked ? "1" : "0");
				upl.set('zip_overwrite', frm.zip_overwrite.checked ? "1" : "0");
				upl.set('zip_keep', frm.zip_keep.checked ? "1" : "0");
			}
			upl.set('characterSet', frm.charset.value);
			upl.set('subtype_id', frm.subtype_id.value);
			upl.set('description', frm.description.value);
			upl.set('the_file', frm.the_file.files[0]);
		
			var url = 'Upload?ID='+App.getSessionId();
			url += '&directory_id='+Data.get('page-data', 'directory-id');
			
			uploadRequest = Ajax.getXmlHttpRequest();
			Ajax.setReadyStateHandler(uploadRequest, upload.uploadComplete, upload.uploadComplete);
			uploadRequest.open('POST', url, true);
			uploadRequest.send(upl);
		
			Doc.hide('selectForm');
			Doc.show('progressForm');
			upload.sendProgressUpdateRequest();
		}
	}
	
	upload.uploadComplete = function(request)
	{
		// If we have called uploadRequest.abort()
		// we do not need error handling here
		if (request.status == 0) return;
		var result = {};
		try
		{
			var contentType = request.getResponseHeader('Content-Type');
			if (request.status != 200 || contentType.indexOf('json') == -1)
			{
				throw new Error(request.responseText);
			}
			result = JSON.parse(request.responseText);
		}
		catch (err)
		{
			result.status = 'error';
			result.message = err;
		}		
		if (result.status == 'ok')
		{
			var url = '../../common/close_popup.jsp?refresh_opener=1&message='+encodeURIComponent(result.message);
			window.top.location = url;
		}
		else
		{
			upload.displayError(result);
		}
	}

	upload.fileOnChange = function()
	{
		var frm = document.forms['file'];
		var filename = frm.the_file.value;
		var isZip = false;
		var zipIndex = 0;
		var longestMatch = 0;
		
		for (var i = 0; i < frm.zip_format.length; i++)
		{
			var zf = frm.zip_format[i].zipFormat;
			var matchLength = upload.matchesZipExtension(zf, filename);
			if (matchLength > longestMatch)
			{
				isZip = true;
				zipIndex = i;
				longestMatch = matchLength;
			}
		}
		
		if (frm.zip_format)
		{
			frm['zip_format'].disabled = !isZip;
			frm['zip_format'].selectedIndex = zipIndex;
			frm['zip_unpack'].disabled = !isZip;
			frm['zip_overwrite'].disabled = !isZip || !frm['zip_unpack'].checked;
			frm['zip_keep'].disabled = !isZip || !frm['zip_unpack'].checked;
			frm['compressed'].disabled = isZip;
			if (isZip)
			{
				// Select '-auto-' if present, otherwise 'no'
				Forms.selectListOption(frm['compressed'], '0');
				Forms.selectListOption(frm['compressed'], '');
			}
		}
	}

	upload.initZipFormats = function()
	{
		var frm = document.forms['file'];
		var zipFormats = Data.json('page-data', 'zip-formats');
		
		for (var i = 0; i < zipFormats.length; i++)
		{
			var zf = zipFormats[i];
			var option = new Option(upload.getFullZipFormatName(zf), zf.id);
			frm.zip_format[frm.zip_format.length] = option;
			option.zipFormat = zf;
		}
	}

	upload.getFullZipFormatName = function(zipFormat)
	{
		var s = zipFormat.name + " [";
		for (var i = 0; i < zipFormat.extensions.length; i++)
		{
			if (i > 0) s += ', ';
			s += '*.' + zipFormat.extensions[i];
		}
		s += ']';
		return s;
	}
	
	upload.matchesZipExtension = function(zipFormat, filename)
	{
		var longestMatch = 0;
		for (var i = 0; i < zipFormat.extensions.length; i++)
		{
			var ext = '.' + zipFormat.extensions[i];
			var index = filename.lastIndexOf(ext);
			if (index > 0 && (index + ext.length == filename.length))
			{
				if (ext.length > longestMatch)
				{
					longestMatch = ext.length;
				}
			}
		}
		return longestMatch;
	}
	
	upload.unpackZipOnClick = function()
	{
		var frm = document.forms['file'];
		var checked = frm['zip_unpack'].checked;
		if (!checked) frm['zip_keep'].checked = true;
		frm['zip_overwrite'].disabled = !checked;
		frm['zip_keep'].disabled = !checked;
	}

	upload.sendProgressUpdateRequest = function()
	{
		var request = Ajax.getXmlHttpRequest();
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetProgress&'+Math.random();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, upload.progressUpdateCallback);
		request.send(null);
	}
	
	upload.progressUpdateCallback = function(request)
	{
		var progress = JSON.parse(request.responseText);
		if (progress)
		{
			if (progress.status == 'ok')
			{
				if (progress.file)
				{
					var filename = progress.file;
					var transferredBytes = progress.transferredBytes;
					var totalBytes = progress.totalBytes;
					var timeInSeconds = progress.timeInSeconds;
					var message = progress.message;
					upload.displayProgress(filename, transferredBytes, totalBytes, timeInSeconds, message);
				}
			}
			else
			{
				upload.displayError(progress);
				uploadRequest.abort(); // Important!! Otherwise the browser may try to re-send the request
				return;
			}
		}
		setTimeout(upload.sendProgressUpdateRequest, 1500);
	}

	
	upload.displayProgress = function(fileName, transferredBytes, totalBytes, transferTime, message)
	{
		// File name
		Doc.element('fileName').innerHTML = 'Uploading ' + fileName + '...';
			
		// Percent done
		var percentDone = totalBytes == 0 ? 0 : Math.round(100 * transferredBytes / totalBytes);
		Doc.element('percentText').innerHTML = percentDone+'%';
		var doneElement = Doc.element('percentDone');
		doneElement.style.width = percentDone+'%';
		var remainElement = Doc.element('percentRemain');
		remainElement.style.width = (100-percentDone)+'%';
		if (percentDone == 100) Doc.hide('percentRemain');
		
		// Bytes
		Doc.element('transferredBytes').innerHTML = Numbers.formatBytes(transferredBytes);
		Doc.element('totalBytes').innerHTML = Numbers.formatBytes(totalBytes);

		// Times and rate
		Doc.element('transferTime').innerHTML = Numbers.formatSeconds(transferTime);
		var transferRate = transferTime == 0 ? 0 : transferredBytes / transferTime;
		Doc.element('transferRate').innerHTML = '(' + Numbers.formatBytes(transferRate) + '/s)';
		if (transferRate > 0)
		{
			var remainTime = parseInt((totalBytes-transferredBytes) / transferRate);
			Doc.element('remainTime').innerHTML = Numbers.formatSeconds(remainTime) + ' (estimated)';
		}
		
		// Message
		if (message)
		{
			Doc.element('message').innerHTML = message.replace(/(.*):(.*)/, '<th>$1</th><td>$2</td>');
		}
		else
		{
			Doc.element('message').innerHTML = '';
		}
	}

	upload.displayError = function(result)
	{
		Doc.show('uploadFailed');
		Doc.hide('progressForm');
		Doc.element('failTitle').innerHTML = result.title || 'File upload failed';
		Doc.element('failMessage').innerHTML = result.message || result.stacktrace;
		if (result.status == 'abort') setTimeout(App.closeWindow, 1500);
	}

	upload.abortUpload = function()
	{
		if (confirm('Are you sure you want to abort the upload? It cannot be resumed.'))
		{
			/*
			Aborting is tricky. See tickets:
			http://base.thep.lu.se/ticket/2018
			http://base.thep.lu.se/ticket/2019
			
			Sending "Abort" command to the server AND 
			uploadRequest.abort() should tell both ends.
			The "abort" is catched by the progress updater that
			also display the error message.
			*/
			var request = Ajax.getXmlHttpRequest();
			var url = 'ajax.jsp?ID='+App.getSessionId();
			url += '&cmd=Abort&'+Math.random();
			request.open("GET", url, true);
			request.send(null);

			uploadRequest.abort();
		}
	}

	return upload;
}();

Doc.onLoad(Upload.initPage);
