<%-- $Id: manage_plates.jsp 7954 2021-05-12 13:27:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.ArrayDesign"
	import="net.sf.basedb.core.ArrayDesignPlate"
	import="net.sf.basedb.core.Plate"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.BaseException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.json.JsonUtil"
	import="net.sf.basedb.util.json.JsonConverter"
	import="org.json.simple.JSONObject"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.Date"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%
final Item itemType = Item.ARRAYDESIGN;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl(":Manage plates");
try
{
	ArrayDesign design = ArrayDesign.getById(dc, itemId);
	design.checkPermission(Permission.WRITE);
	cc.setObject("item", design);
	String title = "Attach plate -- " + HTML.encodeTags(design.getName());
	
	// Load current plates
	JSONObject jsonPlates = new JSONObject();
	jsonPlates.put("itemType", "PLATE");
	ItemQuery<ArrayDesignPlate> platesQuery = design.getArrayDesignPlates();
	platesQuery.include(Include.MINE, Include.SHARED, Include.OTHERS, Include.IN_PROJECT);
	platesQuery.order(Orders.asc(Hql.property("position")));
	jsonPlates.put("items", JsonUtil.toArray(platesQuery.iterate(dc), new JsonConverter<ArrayDesignPlate>()
	{
		public Object convert(ArrayDesignPlate adp)
		{
			Plate plate = adp.getPlate();
			JSONObject json = new JSONObject();
			json.put("id", plate.getId());
			json.put("name", plate.getName());
			return json;
		}
	}));
	%>
	<base:page type="popup" title="<%=title%>">
	<base:head scripts="linkitems-2.js,~manage_plates.js" />
	<base:body>
		<h1><%=title%> <base:help helpid="arraydesign.edit.plates" /></h1>
		<form action="index.jsp?ID=<%=ID%>" method="post" name="design">
		<input type="hidden" name="cmd" value="UpdatePlates">

		<div class="content bottomborder">
			<table class="fullform">
			<tr>
				<th>Array design</th>
				<td><%=HTML.encodeTags(design.getName())%></td>
			</tr>
			<tr class="dynamic">
				<th>Plates</th>
				<td>
					<div class="selectionlist">
					<table>
					<tr style="vertical-align: top;">
					<td>
						<base:buttongroup vertical="true">
							<base:button
								id="moveUp"
								data-down="0"
								image="move_up.png" 
								tooltip="Move up" 
								subclass="square"
							/>
							<base:button 
								id="moveDown"
								data-down="1"
								image="move_down.png" 
								tooltip="Move down" 
								subclass="square"
							/>
						</base:buttongroup>
					</td>
					<td>
						<select name="plates" id="plates" 
							class="auto-init"
							data-auto-init="link-container"
							data-initial-items="[<%=HTML.encodeTags(jsonPlates.toJSONString()) %>]"
							size="15" multiple>
						</select>
					</td>
					<td>
						<base:buttongroup vertical="true">
							<base:button 
								subclass="leftaligned auto-init"
								data-auto-init="add-link"
								data-list-id="plates"
								data-item-type="PLATE"
								style="width: 12em;"
								title="Add plates..." 
								tooltip="Add plates to be used"
							/>
							<base:button 
								subclass="leftaligned auto-init"
								data-auto-init="remove-link"
								data-list-id="plates"
								style="width: 12em;"
								title="Remove" 
								tooltip="Remove the selected plates"
							/>
						</base:buttongroup>
					</td>
					</tr>
					</table>
					</div>
				</td>
			</tr>
			</table>
		</div>
		</form>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
