<%-- $Id: view_well.jsp 7954 2021-05-12 13:27:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Plate"
	import="net.sf.basedb.core.Well"
	import="net.sf.basedb.core.Reporter"
	import="net.sf.basedb.core.ReporterType"
	import="net.sf.basedb.core.ExtendedProperties"
	import="net.sf.basedb.core.ExtendedProperty"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.core.data.ReporterData"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.util.formatter.WellCoordinateFormatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction" 
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.Date"
	import="java.util.Map"
	import="java.util.Set"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="p" uri="/WEB-INF/path.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.WELL;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.ITEM);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl(":View "+itemType);
try
{
	final boolean readReporter = !sc.hasPermission(Permission.DENIED, Item.REPORTER);
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	WellCoordinateFormatter rowFormatter = new WellCoordinateFormatter(true);
	WellCoordinateFormatter columnFormatter = new WellCoordinateFormatter(false);
	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);

	String title = null;
	final Well well = Well.getById(dc, itemId);
	final Plate plate = well.getPlate();
	ReporterData reporter = readReporter ? well.getReporter() : null;
	final int plateId = plate.getId();

	Well parentWell = null;
	boolean readParentWell = true;
	Plate parentWellPlate = null;
	boolean readParentWellPlate = true;
	try
	{
		parentWell = well.getParent();
	}
	catch (PermissionDeniedException ex)
	{
		readParentWell = false;
		readParentWellPlate = false;
	}
	if (parentWell != null)
	{
		try
		{
			parentWellPlate = parentWell.getPlate();
		}
		catch (PermissionDeniedException ex)
		{
			readParentWellPlate = false;
		}
	}
	
	boolean readCurrentReporterType = true;
	ReporterType currentReporterType = null;
	if (reporter != null)
	{
		reporter = Reporter.getById(dc, reporter.getId());
		try
		{
			currentReporterType = Reporter.getReporterType(dc, reporter);
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentReporterType = false;
		}
	}
	
	final boolean writePermission = well.hasPermission(Permission.WRITE);
	final boolean deletePermission = well.hasPermission(Permission.DELETE);
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, guiContext, well);
	ExtensionsInvoker<ButtonAction> invoker = ToolbarUtil.useExtensions(jspContext);
	%>
	<base:page title="<%=title%>" id="view-page">
	<base:head scripts="table.js,tabcontrol-2.js,~wells.js" styles="toolbar.css,table.css,headertabcontrol.css,path.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<p:path><p:pathelement 
			title="Plates" href="<%="../index.jsp?ID="+ID%>" 
			/><p:pathelement 
				title="<%=HTML.encodeTags(plate.getName())%>" 
				href="<%="index.jsp?ID="+ID+"&amp;plate_id="+plateId%>" 
			/><p:pathelement title="<%="Well: "+rowFormatter.format(well.getRow())+columnFormatter.format(well.getColumn())%>" 
			/></p:path>
		<div id="page-data" data-item-id="<%=itemId%>"></div>
		
		<t:tabcontrol 
			id="main" 
			subclass="content mastertabcontrol" 
			active="properties">
		<t:tab id="properties" title="Properties">
			<div>
			<table class="fullform bottomborder">
			<tr>
				<th class="itemstatus"></th>
				<td style="padding: 0px;">
					<tbl:toolbar subclass="bottomborder bg-filled-50">
						<tbl:button 
							id="btnEdit"
							disabled="<%=!writePermission%>" 
							image="edit.png" 
							title="Edit&hellip;" 
							tooltip="<%=writePermission ? "Edit this well" : "You do not have permission to edit this well"%>" 
						/>
						<tbl:button 
							id="btnImport"
							image="import.png" 
							data-plugin-type="IMPORT"  
							title="Import&hellip;" 
							tooltip="Import data" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
						/>
						<tbl:button 
							id="btnExport"
							image="export.png"
							data-plugin-type="EXPORT" 
							title="Export&hellip;" 
							tooltip="Export data" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
						/>
						<tbl:button 
							id="btnRunPlugin"
							image="runplugin.png"  
							data-plugin-type="OTHER" 
							title="Run plugin&hellip;" 
							tooltip="Run a plugin" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
						/>
						<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
							wrapper="<%=new PrefixSuffixRenderer<ButtonAction>(jspContext, "<td>", "</td>") %>"/>
						<tbl:button
							image="help.png"
							subclass="auto-init"
							data-auto-init="help"
							data-help-id="well.view.properties"
							title="Help&hellip;"
							tooltip="Get help about this page"
						/>
					</tbl:toolbar>
				</td>
			</tr>
			<tr>
				<th>Position</th>
				<td><%=rowFormatter.format(well.getRow())%><%=columnFormatter.format(well.getColumn())%></td>
			</tr>
			<tr>
				<th>Parent plate/well</th>
				<td>
					<%
					if (!readParentWell)
					{
						%>
						<i>- denied -</i>
						<%
					}
					else if (parentWell == null)
					{
						%>
						<i>- none -</i>
						<%
					}
					else
					{
						%>
						<%=Base.getLink(ID, parentWellPlate.getName() + " [" + parentWell.getRow() + ", " + parentWell.getColumn() + "]",
							Item.WELL, parentWell.getId(), true) %>
						<%
					}
					%>
				</td>
			</tr>
			<%
			if (reporter == null)
			{
				%>
				<tr>
					<th>Reporter</th>
					<td><i>- <%=readReporter ? "none" : "denied"%> -</i></td>
				</tr>
				<%
			}
			%>
			</table>
			</div>
			
			<%
			if (reporter != null)
			{
				%>
				<table class="fullform bottomborder">
					<tbody class="sectionheader">
						<tr>
							<th colspan="2">Reporter</th>
						</tr>
					</tbody>
					<tr>
						<th>Name</th>
						<td><%=Base.getLink(ID, reporter.getName(), Item.REPORTER, reporter.getId(), true) %></td>
					</tr>
					<tr>
						<th>External ID</th>
						<td><%=HTML.encodeTags(reporter.getExternalId())%></td>
					</tr>
					<tr>
						<th>Type</th>
						<td><base:propertyvalue dbcontrol="<%=dc%>" item="<%=reporter%>" property="reporterType" /></td>
					</tr>
					<tr>
						<th>Registered</th>
						<td><%=dateFormatter.format(reporter.getEntryDate())%></td>
					</tr>
					<tr>
						<th>Last update</th>
						<td><%=dateTimeFormatter.format(reporter.getLastUpdate())%></td>
					</tr>
					<tr>
						<th class="subprompt">- from/by</th>
						<td><%=HTML.niceFormat(reporter.getLastSource())%></td>
					</tr>
					<tr>
						<th>Gene symbol</th>
						<td><%=HTML.encodeTags(reporter.getSymbol())%></td>
					</tr>
					<tr>
						<th>Description</th>
						<td><%=HTML.niceFormat(reporter.getDescription())%></td>
					</tr>
				</table>
				
				<%
				List<ExtendedProperty> reporterProperties = ExtendedProperties.getProperties("ReporterData");
				%>
				<base:section
					id="extendedProperties"
					title="<%="Extended properties (" + reporterProperties.size() + ")"%>"
					context="<%=cc%>"
					>
					<table class="fullform fullborder">
						<%				
						if (reporterProperties != null)
						{
							boolean needsTr = true;
							for (ExtendedProperty ep : reporterProperties)
							{
								String name = ep.getName();
								Formatter<Object> f = FormatterFactory.getExtendedPropertyFormatter(sc, ep);
								String value = f.format(reporter.getExtended(name));
								%>
									<%=needsTr ? "<tr><th>" : "<th class=\"leftborder\">" %>
									<%=HTML.encodeTags(ep.getTitle())%></th>
									<td><%=value%></td>
									<%=needsTr ? "" : "</tr>"%>
								<%
								needsTr = !needsTr;
							}
							%>
							<%=needsTr ? "" : "<th class=\"leftborder\"></th><td></td></tr>"%>
							<%
						}
						%>
					</table>
				</base:section>
				<%
			}
			%>

			<jsp:include page="../../../common/anytoany/list_anytoany.jsp">
				<jsp:param name="ID" value="<%=ID%>" />
				<jsp:param name="item_type" value="<%=itemType.name()%>" />
				<jsp:param name="item_id" value="<%=itemId%>" />
				<jsp:param name="title" value="Other items related to this well" />
			</jsp:include>
			</t:tab>
			<t:tab id="annotations" title="Annotations" 
				tooltip="View annotation values" clazz="white">
				<jsp:include page="../../../common/annotations/list_frameset.jsp">
					<jsp:param name="item_type" value="<%=itemType.name()%>" />
					<jsp:param name="item_id" value="<%=itemId%>" />
					<jsp:param name="ID" value="<%=ID%>" />
				</jsp:include>
			</t:tab>
		</t:tabcontrol>

	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
