<%-- $Id: edit_reporter.jsp 7954 2021-05-12 13:27:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.ExtendedProperties"
	import="net.sf.basedb.core.ExtendedProperty"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.Reporter"
	import="net.sf.basedb.core.ReporterType"
	import="net.sf.basedb.core.data.ReporterData"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.util.json.JsonUtil"
	import="net.sf.basedb.util.json.JsonConverter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.formatter.FormatterSettings"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.tabcontrol.TabAction"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.Date"
	import="java.util.List"
	import="org.json.simple.JSONArray"
	import="org.json.simple.JSONObject"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.REPORTER;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl(":Edit "+itemType);
ItemResultIterator<ReporterType> reporterTypes = null;
try
{
	List<ExtendedProperty> reporterProperties = ExtendedProperties.getProperties("ReporterData");
	JSONArray jsonReporterProperties = JsonUtil.toArray(reporterProperties, new JsonConverter<ExtendedProperty>()
		{
			public Object convert(ExtendedProperty ep)
			{
				JSONObject json = new JSONObject();
				json.put("name", ep.getName());
				json.put("valueType", ep.getType().name());
				json.put("title", ep.getTitle());
				json.put("nullable", ep.isNullable());
				return json;
			}
		});
	
	String title = null;
	ReporterData reporter = null;
	boolean readCurrentReporterType = true;
	int currentReporterTypeId = 0;
	
	if (itemId == 0)
	{
		int recentType = Values.getInt(cc.getRecent(Item.REPORTERTYPE.name(), 0), 0);
		currentReporterTypeId = Values.getInt(cc.getPropertyValue("reporterType"), recentType);
		title = "Create reporter";
		cc.removeObject("item");
	}
	else
	{
		reporter = Reporter.getById(dc, itemId);
		title = "Edit reporter -- " + HTML.encodeTags(reporter.getName());
		try
		{
			ReporterType rt = Reporter.getReporterType(dc, reporter);
			if (rt != null) 
			{
				currentReporterTypeId = rt.getId();
			}
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentReporterType = false;
		}
		cc.setObject("item", reporter);
	}
	if (!sc.hasPermission(Permission.WRITE, itemType))
	{
		throw new PermissionDeniedException(Permission.WRITE, itemType.toString());
	}
	
	// Query to retrieve reporter types
	final ItemQuery<ReporterType> reporterTypeQuery = ReporterType.getQuery();
	reporterTypeQuery.order(Orders.asc(Hql.property("name")));
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	String dateFormat = FormatterSettings.getDateFormat(sc);
	String jsDateFormat = HTML.javaScriptEncode(dateFormat);
	String htmlDateFormat = HTML.encodeTags(dateFormat);
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	String dateTimeFormat = FormatterSettings.getDateTimeFormat(sc);
	String jsDateTimeFormat = HTML.javaScriptEncode(dateTimeFormat);
	String htmlDateTimeFormat = HTML.encodeTags(dateTimeFormat);
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), reporter);
	ExtensionsInvoker<TabAction> invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,~reporters.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>
		
		<div id="page-data" class="datacontainer"
			data-date-format="<%=htmlDateFormat%>"
			data-datetime-format="<%=htmlDateTimeFormat%>"
			data-extended-properties="<%=HTML.encodeTags(jsonReporterProperties.toJSONString())%>"
			></div>
		
		<form action="index.jsp?ID=<%=ID%>" method="post" name="reporter">
		<input type="hidden" name="cmd" value="UpdateItem">
		
		<t:tabcontrol id="settings" 
			subclass="content dialogtabcontrol"
			position="bottom" remember="<%=reporter != null%>"
			extensions="<%=invoker%>">
		<t:tab id="info" title="Reporter" helpid="reporter.edit">
			<table class="fullform input100">
			<tr>
				<th>Name</th>
				<td><input class="text required auto-init" data-auto-init="<%=reporter == null ? "focus-select" : "focus" %>" 
					type="text" name="name" 
					value="<%=HTML.encodeTags(reporter == null ? Values.getString(cc.getPropertyValue("name"), "New reporter") : reporter.getName())%>" 
					maxlength="<%=ReporterData.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>External ID</th>
				<td><input class="text required" type="text" name="externalId" 
					value="<%=HTML.encodeTags(reporter == null ? cc.getPropertyValue("externalId") : reporter.getExternalId())%>" 
					maxlength="<%=ReporterData.MAX_EXTERNAL_ID_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Type</th>
				<td>
					<select name="reportertype_id" <%=!readCurrentReporterType ? "disabled readonly class=\"disabled\"" : ""%> style="width: 15em;">
					<%
					if (!readCurrentReporterType)
					{
						%>
						<option value="-1">- denied -
						<%
					}
					else
					{
						%>
						<option value="0">- none -
						<%
						reporterTypes = reporterTypeQuery.iterate(dc);
						while (reporterTypes.hasNext())
						{
							ReporterType reporterType = reporterTypes.next();
							int rtId = reporterType.getId();
							%>
							<option
								value="<%=rtId == currentReporterTypeId && reporter != null ? -rtId : rtId %>"
								<%=rtId == currentReporterTypeId ? "selected" : "" %>
								><%=HTML.encodeTags(reporterType.getName())%>
							<%
						}
					}
					%>
					</select>
				</td>
				<td></td>
			</tr>
			<tr>
				<th>Gene symbol</th>
				<td><input class="text" type="text" name="symbol" 
					value="<%=HTML.encodeTags(reporter == null ? cc.getPropertyValue("symbol") : reporter.getSymbol())%>"
					maxlength="<%=ReporterData.MAX_SYMBOL_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="6" name="description" id="description"
						><%=HTML.encodeTags(reporter == null ? cc.getPropertyValue("description") : reporter.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			</table>
			
		</t:tab>
		
		<t:tab id="extended" title="Extended properties" 
			helpid="reporter.edit.extended">
			<table class="fullform input100">
				<%
				if (reporterProperties != null)
				{
					for (ExtendedProperty ep : reporterProperties)
					{
						String name = ep.getName();
						String fieldName = "ep."+name;
						Type type = ep.getType();
						boolean required = !ep.isNullable();
						Object value = reporter == null ? cc.getPropertyValue(name) : reporter.getExtended(name);
						String theClazz = required ? "text required" : "text";
						boolean hasZoom = false;
						%>
						<tr>
							<th><%=HTML.encodeTags(ep.getTitle())%></th>
							<td>
							<%
							if (type == Type.INT || type == Type.LONG)
							{
								%>
								<input class="<%=theClazz%>" type="text" name="<%=fieldName%>" id="<%=fieldName%>"
									value="<%=value == null ? "" : value%>" style="width: 15em;" maxlength="20">
								<%
							}
							else if (type == Type.FLOAT || type == Type.DOUBLE)
							{
								%>
								<input class="<%=theClazz%>" type="text" name="<%=fieldName%>" id="<%=fieldName%>"
									value="<%=value == null ? "" : value%>" style="width: 15em;" maxlength="20">
								<%
							}
							else if (type == Type.STRING)
							{
								%>
								<input class="<%=theClazz%>" type="text" name="<%=fieldName%>" id="<%=fieldName%>"
									value="<%=HTML.encodeTags((String)value)%>"
									maxlength="<%=ep.getLength()%>"
									>
								<%
							}
							else if (type == Type.TEXT)
							{
								hasZoom = true;
								%>
								<textarea class="<%=theClazz%>" name="<%=fieldName%>" id="<%=fieldName%>" rows="6" 
									><%=HTML.encodeTags((String)value)%></textarea>
								<%
							}
							else if (type == Type.BOOLEAN)
							{
								Boolean b = (Boolean)value;
								if (!required)
								{
									%>
									<input type="radio" name="<%=fieldName%>" id="<%=fieldName%>.null" value=""
										<%=b == null ? "checked" : ""%> 
										><label for="<%=fieldName%>.null"><i>- not specified -</i></label><br>
									<%
								}
								%>
								<input type="radio" name="<%=fieldName%>" id="<%=fieldName%>.true" value="true" 
									<%=b != null && b == true ? "checked" : ""%>
									><label for="<%=fieldName%>.true">true</label><br>
								<input type="radio" name="<%=fieldName%>" id="<%=fieldName%>.false" value="false" 
									<%=b != null && b == false ? "checked" : ""%>
									><label for="<%=fieldName%>.false">false</label>
								<%
							}
							else if (type == Type.DATE)
							{
								%>
								<table>
								<tr>
								<td>
									<input class="<%=theClazz%>" type="text" name="<%=fieldName%>" id="<%=fieldName%>"
										value="<%=dateFormatter.format((Date)value)%>" style="width: 15em;"
										maxlength="20" title="Enter date in format: <%=htmlDateFormat%>"
										>
								</td>
								<td>
									<base:calendar textarea="<%=fieldName%>" title="<%=HTML.encodeTags(ep.getTitle()) %>" />
								</td>
								</tr>
								</table>
								<%
							}
							else if (type == Type.TIMESTAMP)
							{
								%>
								<table>
								<tr>
								<td>
									<input class="<%=theClazz%>" type="text" name="<%=fieldName%>" id="<%=fieldName%>"
										value="<%=dateTimeFormatter.format((Date)value)%>" style="width: 15em;"
										maxlength="20" title="Enter timestamp in format: <%=htmlDateTimeFormat%>"
										>
								</td>
								<td>
									<base:calendar textarea="<%=fieldName%>" title="<%=HTML.encodeTags(ep.getTitle())%>"  
										data-use-time="1"
										tooltip="Select a timestamp from a calendar" 
									/>
								</td>
								</tr>
								</table>
								<%
							}
							%>
							</td>
							<td style="width: 20px;">
								<%
								if (hasZoom)
								{
									%>
									<base:zoom textarea="<%=fieldName%>" title="<%=HTML.encodeTags(ep.getTitle()) %>" />
									<%
								}
								%>
							</td>
						</tr>
						<%
					}
				}
				%>
				<tr class="dynamic">
					<th></th>
					<td></td>
					<td></td>
				</tr>
			</table>
		</t:tab>
		</t:tabcontrol>
		</form>
		
		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>

	</base:body>
	</base:page>
	<%
}
finally
{
	if (reporterTypes != null) reporterTypes.close();
	if (dc != null) dc.close();
}

%>
